// FileChangeEvent.cpp: implementation of the CFileChangeEvent class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FileChangeEvent.h"
#include "FileInfo.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFileChangeEvent::CFileChangeEvent()
{
	m_bEvent = false;
  m_hWatchEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
  m_hThreadTermEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
  m_hThread = NULL;
}

CFileChangeEvent::~CFileChangeEvent()
{
	if ( m_bEvent )
		stopWatch();

	for (FileInfoMap::iterator it = m_FileInfoMap.begin(); it != m_FileInfoMap.end(); it++)
		delete it->second;
	m_FileInfoMap.clear();
	for (FilePathVector::iterator it2 = m_Paths.begin(); it2 != m_Paths.end(); it2++)
		delete *it2;
	m_Paths.clear();
  CloseHandle(m_hWatchEvent);
  CloseHandle(m_hThreadTermEvent);
  m_hWatchEvent = NULL;
  m_hThreadTermEvent = NULL;
}

void CFileChangeEvent::startWatch(void)
{
	if ( ! m_bEvent )
	{
    ResetEvent(m_hWatchEvent);
		m_bEvent = (NULL != (m_hThread = CreateThread(NULL, 0, FileChangeWatch,(LPVOID)this, 0, &m_dwThreadID)));
	}
}

void CFileChangeEvent::stopWatch(void)
{
	if ( m_bEvent )
	{
		m_bEvent = false;
    SetEvent(m_hWatchEvent);
    WaitForSingleObject(m_hThreadTermEvent, INFINITE);
    CloseHandle(m_hThread);
    m_hThread = NULL;
	}
}

void CFileChangeEvent::addFile(const std::string &sFileName)
{
	if ( m_FileInfoMap.find(sFileName) != m_FileInfoMap.end() )
		return;

	CFileInfo *f = new CFileInfo(sFileName);

	m_FileInfoMap.insert(std::make_pair(sFileName, f));

	for (FilePathVector::iterator it = m_Paths.begin(); it != m_Paths.end(); it++)
	{
		if ( (*it)->getName() == f->getPath() )
		{
			(**it)++;
			return;
		}
	}
	m_Paths.push_back(new CFilePath(f->getPath()));

	if ( m_bEvent )
	{
		stopWatch();
		startWatch();
	}
}

void CFileChangeEvent::removeFile(const std::string &sFileName)
{
	CFileInfo f(sFileName);

	for (FilePathVector::iterator it = m_Paths.begin(); it != m_Paths.end(); it++)
	{
		if ( (*it)->getName() == f.getPath() )
		{
			(**it)--;
			if ( (*it)->getCount() <= 0 )
			{
				delete *it;
				m_Paths.erase(it);
			}
			return;
		}
	}
}

DWORD WINAPI CFileChangeEvent::FileChangeWatch(LPVOID lpParameter)
{
	CFileChangeEvent *pFileChangeEvent = (CFileChangeEvent *) lpParameter;

  int nNumItems = pFileChangeEvent->m_Paths.size();
  HANDLE *hChangeHandles = new HANDLE[nNumItems + 1];
 
	while ( 1 )	//endless loop
	{
    hChangeHandles[0] = pFileChangeEvent->m_hWatchEvent;

    for (int i = 0; i < nNumItems; i++)
		{
      char* szName = (char*) pFileChangeEvent->m_Paths[i]->getName().c_str();

      hChangeHandles[i+1] = FindFirstChangeNotification(szName, 
															  FALSE,
															  FILE_NOTIFY_CHANGE_LAST_WRITE
															  | FILE_NOTIFY_CHANGE_SIZE
															  | FILE_NOTIFY_CHANGE_FILE_NAME
//                                | FILE_NOTIFY_CHANGE_DIR_NAME
															 );
		}
    
    DWORD dwResult = WaitForMultipleObjects(nNumItems + 1, hChangeHandles, FALSE, INFINITE);

		if ( (dwResult <= WAIT_OBJECT_0) || (dwResult > WAIT_OBJECT_0 + nNumItems) ) // Watch stopped 
		{
			break;
		}
		
		int nPathIdx = dwResult - WAIT_OBJECT_0 - 1;

		for (FileInfoMap::iterator it = pFileChangeEvent->m_FileInfoMap.begin(); it != pFileChangeEvent->m_FileInfoMap.end(); it++)
		{
      const char* p1 = it->second->getPath().c_str();
      const char* p2 = pFileChangeEvent->m_Paths[nPathIdx]->getName().c_str();
			if ( it->second->getPath() == pFileChangeEvent->m_Paths[nPathIdx]->getName() )
			{
        WIN32_FIND_DATA stFindData;
        char szItem[MAX_PATH];
        lstrcpy(szItem, it->second->getName().c_str());

        int nLen = lstrlen(szItem);
        BOOL bIsFolder = FALSE;

        if (nLen && '\\' == szItem[nLen - 1])
        {
          szItem[nLen - 1] = '\0';
          bIsFolder = TRUE;
        }

        HANDLE f = FindFirstFile(szItem, &stFindData);

				if (INVALID_HANDLE_VALUE != f)
				{
					FindClose(f);
					if (-1 == CompareFileTime(&(it->second->getDate()), &(stFindData.ftLastWriteTime)))
					{
						it->second->setDate(stFindData.ftLastWriteTime);
						pFileChangeEvent->OnFileAlarm(FA_CHANGED, it->second->getName().c_str());
					}
          else
          if (bIsFolder)
          {
						pFileChangeEvent->OnFileAlarm(FA_CHANGED, it->second->getName().c_str());
          }
				}
				else
				{
          pFileChangeEvent->removeFile(it->second->getName().c_str());
					pFileChangeEvent->OnFileAlarm(FA_REMOVED, it->second->getName().c_str());
				}
			}
		}
  }

  for (int i = 0; i < nNumItems; i++)
    FindCloseChangeNotification(hChangeHandles[i+1]);

	delete[] hChangeHandles;
  SetEvent(pFileChangeEvent->m_hThreadTermEvent);
	return 0;
}
